#include <stdio.h>
#include <GLUT/glut.h>
#include <stdlib.h>
#include <math.h>

#define RND ((float)rand( )/RAND_MAX)

#define True  1
#define False 0
#define WINSIZE 1000
#define FRACTALSIZE 4000
#define MAX 100

void display(void);
void idle(void);
void init(void);
int  main(int argc, char** argv);

float fractal[FRACTALSIZE][FRACTALSIZE];
float orbit[2*MAX*FRACTALSIZE];
float y;

float distance(float x, float y)
{
    float i = pow(x, 2.0) + pow(y, 2.0);
    i       = sqrt(i);
    return i;
}

int mandel(float zx, float zy, float cx, float cy, int count)
{
    float new_zx, new_zy;

    if(distance(zx, zy) > 10)
    {
        orbit[2*count] = -1000.0;
		return 0;
    }
    else if(count > MAX)
    {
		orbit[2*count] = -1000.0;
        return 1;
    }
	orbit[2*count] = zx;
	orbit[2*count+1] = zy;
	
    new_zx = pow(zx, 2.0) - pow(zy, 2.0) + cx;
    new_zy = 2*zx*zy + cy;
    return mandel(new_zx, new_zy, zx, zy, count + 1);
}

void create_fractal( )
{
	int i, j, k, out;
	int zx, zy;
	
	for(i = 0; i < FRACTALSIZE; i++)
	{
		for(j = 0; j < FRACTALSIZE; j++)
		{
			fractal[i][j] = 0;
		}
		orbit[2*i] = 0;
		orbit[2*i+1] = 0;
	}
	
	for(i = 0; i < FRACTALSIZE; i++)
	{
		for(j = 0; j < FRACTALSIZE; j++)
		{
			out = mandel(0,0, i*1.0/FRACTALSIZE, j*1.0/FRACTALSIZE, 0);
			if(out == 1)
			{
				k = 0;
				while(orbit[k] != -1000.0)
				{
					zx = floor(orbit[2*k]*FRACTALSIZE);
					zy = floor(orbit[2*k+1]*FRACTALSIZE);
					//printf("%d %d\n", zx, zy);
					if(zx > 0 && zy > 0 && zy < FRACTALSIZE && zx < FRACTALSIZE)
					{
						fractal[zx][zy] += 1.0;
					}
					k++;
				}
			}
		}
		printf("%d\n", i);
	}
}

void display( )
{
    int i, j, color;
	glClear(GL_DEPTH_BUFFER_BIT);
    glPointSize(1.0);
	glEnable(GL_POINT_SMOOTH);
    glBegin(GL_POINTS);
        for(i = 0; i < FRACTALSIZE; i++)
        {
			for(j = 0; j < FRACTALSIZE; j++)
			{
				color = fractal[i][j];
				//color = 1.0;
				if(color == 0)
				{
					glColor3f(0.1,0.0,0.0);
				}
				else
				{
					glColor3f(0.1 + 1.0/color, 1.0/color, 1.0/color);
				}
				glVertex3f(j*0.5, y*0.5, 0.0);
			}
			y += 0.5;
        }
    glEnd( );
	glDisable(GL_POINT_SMOOTH);
    glutSwapBuffers( );
    
}

void init( )
{
    glClearColor(0.0, 0.0, 0.0, 0.0);
    glMatrixMode(GL_FLAT);
	glShadeModel (GL_SMOOTH);
    glLoadIdentity( );
    glOrtho(0.0, WINSIZE, 0.0, WINSIZE, -1.0, 1.0);
    glClear(GL_COLOR_BUFFER_BIT);
    y = 0;
	create_fractal( );
}

int main(int argc, char** argv)
{
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
    glutInitWindowSize(WINSIZE, WINSIZE);
    glutInitWindowPosition(0,0);
    glutCreateWindow("MandelBrot");
    init( );
    glutDisplayFunc(display);
    glutMainLoop( );
    return 0;
}
